package cn.jisol.game.service.impl;

import cn.jisol.game.data.ResourceId;
import cn.jisol.game.entity.game.Player;
import cn.jisol.game.entity.game.Resource;
import cn.jisol.game.mapper.ResourceMapper;
import cn.jisol.game.service.ResourceService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.retry.RetryException;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Service
public class ResourceServiceImpl extends ServiceImpl<ResourceMapper, Resource> implements ResourceService {

    /**
     * @param playerId 玩家Id
     * @param type  资源类型
     * @param value 添加的数量
     * @return
     */
    @Override
    @Retryable(value = RetryException.class)
    public Resource addResourceValue(Long playerId, int type, Long value) {

        //获取玩家资源
        Resource res = this.getOne(
                Wrappers.lambdaQuery(Resource.class)
                        .eq(Resource::getPlayerId, playerId) //玩家
                        .eq(Resource::getResourceTbId, type) //资源
        );

        //如果没有资源则默认给一个0的资源
        if(Objects.isNull(res)){
            //创建初始资源
            res = this.initResource(playerId,type);
        }

        if(value > 0){

            //如果是累加 则 执行累加sql
            if(this.update(
                    Wrappers.lambdaUpdate(Resource.class)
                            .eq(Resource::getPlayerId, playerId) //玩家
                            .eq(Resource::getResourceTbId, type) //资源
                            .setSql("resource_value = resource_value + "+value)
            )){
                //添加资源
                res.setResourceValue(res.getResourceValue() + value);
                return res;
            }else{
                throw new RetryException("");
            }

        }else{

            //添加资源
            res.setResourceValue(res.getResourceValue() + value);

            if(res.getResourceValue() <= 0){
                return null;
            }

            //保存资源 (保存失败重试)
            if (!this.saveOrUpdate(res)) throw new RetryException("");

            return res;
        }

    }

    //初始化某个资源
    @Override
    public Resource initResource(Long playerId, int type) {
        Resource resource = Resource.builder().playerId(playerId).resourceTbId(type).resourceValue(0L).build();
        this.initResource(resource);
        return resource;
    }

    @Override
    public Resource initResource(Resource resource) {
        this.saveOrUpdate(resource);
        return resource;
    }


}
